#ifndef STLINK_VERSION_H_
#define STLINK_VERSION_H_

#define STLINK_VERSION       "1.7.0"
#define STLINK_VERSION_MAJOR 1
#define STLINK_VERSION_MINOR 7
#define STLINK_VERSION_PATCH 0

#endif // STLINK_VERSION_H_
